/**
 * Copyright (c) 2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jbi.messaging.exchange;

import java.io.Serializable;
import java.net.URI;
import java.util.Map;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;

/**
 * 
 * An interface to better specify the Petals-specificities of our implementation of {@link MessageExchange}.
 * 
 * This is not meant to be used by components but only inside the container.
 * 
 * @author Victor Noël - Linagora
 *
 */
public interface PetalsMessageExchange extends MessageExchange, Serializable {

    /**
     * See {@link #setDoChecks(boolean)}.
     */
    public static final String PROPERTY_DONT_DO_CHECKS = "org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl.dontDoChecks";

    public static final String IN_MSG = "in";

    public static final String OUT_MSG = "out";

    public static final String FAULT_MSG = "fault";

    public ServiceEndpoint getConsumerEndpoint();

    // TODO mave that as an initialization parameter?
    public void setConsumerEndpoint(final ServiceEndpoint serviceEndpoint);

    // TODO move that as an initialization parameter?
    public void setPattern(final URI pattern);

    /**
     * Direct access to the messages
     */
    public Map<String, NormalizedMessage> getMessages();

    public void setTermination(final String termination);

    /**
     * A message is set to terminated once it has been sent with a non-ACTIVE status or if a timeout occured.
     * 
     * It is useful for {@link DeliveryChannelImpl} to know if someone is not trying to send back an exchange that has
     * been received as non-ACTIVE or has timed out.
     * 
     */
    public String getTermination();

    /**
     * Disable checks about roles, patterns, etc.
     * 
     * It can be useful in the container, but for now only the CKD exploits it.
     * 
     * Another way of disabling checks is setting the property {@value #PROPERTY_DONT_DO_CHECKS}.
     * 
     * @param doChecks
     */
    public void setDoChecks(final boolean doChecks);

    public void setRole(final Role role);
}
